/* Copyright (c) 2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.junit.lib;

import java.io.File;
import java.util.Arrays;
import org.jabhts.junit.tests.aSimpleContent.SimpleContent;
import org.jabhts.runtime.lib.TemplateDataLoader;
import org.jabhts.translator.compiler.htmlLoader.Html5lib;
import org.jabhts.translator.compiler.utils.CompilerUtils;
import org.jabhts.translator.compiler.utils.Ref;

public class TestMain
{
	static int successCount, failCount;
	static final Ref<Html5lib> html5lib = new Ref<Html5lib>();

	public static void main(String[] args)
	{
		try {
			if (args.length < 1)
				throw new Exception("Missing scan dir argument");
			if (args.length < 2)
				throw new Exception("Missing jannotemp-data dir argument");
			File scanDirF = new File(args[0]);
			System.err.println("TestMain: scans " + scanDirF);
			TemplateDataLoader.loader = new TemplateDataLoader.UsingFilesystem(new File(args[1]));
			String s = SimpleContent.class.getPackage().getName();
			packageFilter = s.substring(0, s.lastIndexOf('.') + 1);
			TestBase.baseDirF = scanDirF;
			scan(scanDirF, "");
			System.err.println("Result: " + failCount + " failed, " + successCount + " OK.");
			if (failCount == 0)
				System.exit(0);
		}
		catch (Exception ex) {
			System.err.println(CompilerUtils.getSomeMessage(ex));
			ex.printStackTrace();
		}
		finally {
			if (html5lib.get() != null)
				html5lib.get().close();
		}
		System.exit(1);
	}

	static String packageFilter;

	public static void scan(File dirF, String pkg) throws Exception
	{
		File[] entries = dirF.listFiles();
		if (entries == null || entries.length == 0)
			return;
		Arrays.sort(entries);
		for (File f : entries) {
			String name = f.getName();
			if (f.isDirectory()) {
				scan(f, CompilerUtils.dotJoin(pkg, name));
				continue;
			}
			if (!f.isFile())
				continue;
			if (!pkg.startsWith(packageFilter))
				continue;
			if (!name.endsWith(".java") || name.startsWith(".") || name.startsWith("_"))
				continue;
			runTest(CompilerUtils.dotJoin(pkg, name.substring(0, name.length() - 5)));
		}
	}

	public static void runTest(String className) throws Exception
	{
		Class<?> cls = Class.forName(className);
		if (!TestBase.class.isAssignableFrom(cls))
			return;
		System.err.println("runTest: " + CompilerUtils.zapPrefix(className, packageFilter));
		//noinspection UnusedCatchParameter
		try {
			TestBase b = (TestBase)cls.newInstance();
			b.test();
			successCount++;
			//System.out.print("+");
			//System.out.flush();
		}
		catch (SilentTestException ex) {
			failCount++;
		}
		catch (Exception ex) {
			//System.out.println();
			//System.out.flush();
			System.err.println(className + " FAILED: " + CompilerUtils.getSomeMessage(ex));
			ex.printStackTrace();
			failCount++;
		}
	}
}
