/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.junit.tests.lScope;

import org.jabhts.annotation.element.EHtml;
import org.jabhts.annotation.element.EText;
import org.jabhts.annotation.flowControl.EScopeChilds;
import org.jabhts.annotation.global.TCompile;
import org.jabhts.junit.lib.TestBase;

@TCompile(filename = "TestScope")
public class TestScopeInstance extends TestBase
{
	@EText
	String getTextOne()
	{
		return "Making a test";
	}

	@EScopeChilds
	class BlockOne
	{
		final String msg1;

		BlockOne() throws Exception
		{
			throw new Exception("Should not be called");
		}

		BlockOne(String msg1)
		{
			this.msg1 = msg1;
		}

		@EText
		String getString1()
		{
			return msg1;
		}

		@EHtml
		void printString2()
		{
			pw.htmlEncode("scope-1-print");
		}
	}

	BlockOne newBlockOne()
	{
		return new BlockOne("scope-1-get");
	}

	@EScopeChilds
	static class BlockTwo
	{
		final String msg1;

		BlockTwo() throws Exception
		{
			throw new Exception("Should not be called");
		}

		BlockTwo(String msg1)
		{
			this.msg1 = msg1;
		}

		@EText
		String getString1()
		{
			return msg1;
		}
	}

	BlockTwo newBlockTwo()
	{
		return new BlockTwo("scope-2-get");
	}

	@EScopeChilds("StrongContainer")
	static class Severance
	{
		final String msg1;

		Severance() throws Exception
		{
			throw new Exception("Should not be called");
		}

		Severance(String msg1)
		{
			this.msg1 = msg1;
		}

		@EText(tag = "strong")
		String mymy()
		{
			return msg1;
		}
	}

	static Severance newSeverance()
	{
		return new Severance("Testing gone amuck");
	}

	protected void emitTemplate() throws Exception
	{
		htmlBasename = "TestScope";
		new TestScopeInstanceTp(this).emit(pw);
	}
}
