/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.runtime.lib;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;

public class TemplateData
{
	public final String[] strings;
	public final int[] opcodes;
	public final String sourceMD5;

	public static final String headerMagic = "TmplData";
	public static final String trailerMagic = "({end})";
	public static final int version = 2;

	public TemplateData(String[] strings, int[] opcodes, String sourceMD5)
	{
		this.strings = strings;
		this.opcodes = opcodes;
		this.sourceMD5 = sourceMD5;
	}

	public TemplateData(InputStream is) throws IOException
	{
		DataInputStream dis = new DataInputStream(is);
		String s = dis.readUTF();
		if (!headerMagic.equals(s))
			throw new IOException("Invalid magic - not a template data file: '" + s + "'");
		int v = dis.readShort();
		if (v != version)
			throw new IOException("Invalid version, expected " + version + " but is " + v);
		//
		sourceMD5 = dis.readUTF();
		//
		opcodes = new int[dis.readInt()];
		for (int i = 0; i < opcodes.length; i++)
			opcodes[i] = dis.readInt();
		//
		strings = new String[dis.readInt()];
		for (int i = 0; i < strings.length; i++)
			strings[i] = dis.readUTF();
		//
		s = dis.readUTF();
		if (!trailerMagic.equals(s))
			throw new IOException("TemplateData failed reading trailer marker.");
	}

	public byte[] toBytes(String fullClassName) throws IOException
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try (DataOutputStream os = new DataOutputStream(baos)) {
			os.writeUTF(headerMagic);
			os.writeShort(version);
			//
			os.writeUTF(sourceMD5);
			//
			os.writeInt(opcodes.length);
			for (int i : opcodes)
				os.writeInt(i);
			//
			os.writeInt(strings.length);
			for (String s : strings)
				os.writeUTF(s);
			//
			os.writeUTF(trailerMagic);
		}
		return baos.toByteArray();
	}
}
