/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.element;

import org.jabhts.translator.compiler.application.baseclass.Application;
import org.jabhts.translator.compiler.application.scope.Scope;
import org.jabhts.translator.compiler.tree.AbstractEmitter;
import org.jabhts.translator.compiler.tree.ElementEmitters;
import org.jabhts.translator.compiler.utils.CompilerException;

public abstract class ElementLoopEmitter<AP extends Application> extends AbstractEmitter<AP>
{
	/**
	 * false for loop constructs to avoid duplicate id attributes in generated output.
	 */
	public final boolean allowIdAttributeInSubTree;
	public final Scope subScope;

	protected ElementLoopEmitter(ElementEmitters emitters, AP handler,
	                             boolean allowIdAttributeInSubTree,
	                             Scope subScope)
	{
		super(emitters.program, handler);
		this.allowIdAttributeInSubTree = allowIdAttributeInSubTree;
		this.subScope = subScope;
		if (emitters.alreadyDefined(emitters.eLoop, this, "@EIterate/@EWhile/@EScopingWhile"))
			return;
		emitters.eLoop = this;
		if (!emitters.enterSubScope(this, subScope))
			return;
		emitters.allowIdAttribute &= allowIdAttributeInSubTree;
	}

	/**
	 * Called before tag has been emitted.
	 */
	public abstract void preElement() throws CompilerException;

	/**
	 * Called after the compiler has processed the element.
	 * Normally used for patching goto's for if/while constructs.
	 */
	public abstract void postElement() throws CompilerException;
}
