/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.element.simple;

import org.jabhts.annotation.flowControl.ETagIf;
import org.jabhts.translator.compiler.application.baseclass.AbstractBooleanApplication;
import org.jabhts.translator.compiler.application.scope.TargetingInfo;
import org.jabhts.translator.compiler.codeGeneration.api.PatchedGoto;
import org.jabhts.translator.compiler.tree.AbstractEmitter;
import org.jabhts.translator.compiler.tree.ElementEmitters;
import org.jabhts.translator.compiler.utils.CompilerException;
import org.w3c.dom.Element;

public class ETagIfApplication extends AbstractBooleanApplication
{
	public ETagIfApplication(TargetingInfo target) throws CompilerException
	{
		super(target);
	}

	public void createEmitter(ElementEmitters emitters, Element e) throws CompilerException
	{
		ETagIf a = (ETagIf)annotation;
		new Emitter(emitters, a.inverse());
	}

	public class Emitter extends AbstractEmitter<AbstractBooleanApplication>
	{
		PatchedGoto pgOpen, pgClose;
		final boolean inverse;

		protected Emitter(ElementEmitters emitters, boolean inverse)
		{
			super(emitters.program, ETagIfApplication.this);
			this.inverse = inverse;
			emitters.eTag.add(this);
		}

		public void preElementOpenTag() throws CompilerException
		{
			handler.emitBooleanCallback(program);
			if (!inverse)
				program.addNegateBoolean();
			pgOpen = program.addPatchedConditionalGoto();
		}

		public void postElementOpenTag() throws CompilerException
		{
			pgOpen.patchToCurrentAddress();
		}

		public void preElementCloseTag() throws CompilerException
		{
			handler.emitBooleanCallback(program);
			if (!inverse)
				program.addNegateBoolean();
			pgClose = program.addPatchedConditionalGoto();
		}

		public void postElementCloseTag() throws CompilerException
		{
			pgClose.patchToCurrentAddress();
		}
	}
}
