/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.codeGeneration.util;

import java.io.PrintWriter;

public class JavaPrinter
{
	public final PrintWriter pw;
	public int indent = 0;
	public int singleLineIndent;
	public StringBuilder currentLineSB = new StringBuilder();

	public JavaPrinter(PrintWriter pw, int indent)
	{
		this.pw = pw;
		this.indent = indent;
	}

	public void println()
	{
		eol();
	}

	public void println(String s)
	{
		print(s);
		eol();
	}

	public void print(String s)
	{
		if (s == null)
			return;
		for (int i = 0; i < s.length(); i++) {
			char ch = s.charAt(i);
			if (ch == '\n')
				eol();
			else
				currentLineSB.append(ch);
		}
	}

	public void printEscapedString(String s)
	{
		if (s == null) {
			print("null");
			return;
		}
		currentLineSB.append('"');
		for (int i = 0; i < s.length(); i++) {
			char ch = s.charAt(i);
			switch (ch) {
				case '\n' -> currentLineSB.append("\\n");
				case '\r' -> currentLineSB.append("\\r");
				case '\t' -> currentLineSB.append("\\t");
				case '"', '\\' -> {
					currentLineSB.append('\\');
					currentLineSB.append(ch);
				}
				default -> {
					if ((ch >= 32 && ch <= 126) || ch > 160 && ch < 255) {
						currentLineSB.append(ch);
						break;
					}
					if (ch <= 255)
						currentLineSB.append(String.format("\\x%02x", (int)ch));
					else
						currentLineSB.append(String.format("\\u%04x", (int)ch));
				}
			}
		}
		currentLineSB.append('"');
	}

	private void eol()
	{
		if (currentLineSB.length() == 0) {
			pw.println();
			return;
		}
		String s = currentLineSB.toString();
		currentLineSB.setLength(0);
		if (s.startsWith("}") || s.endsWith("}") || s.endsWith("};")) {
			if (!s.contains(" { ")) {
				indent--;
				singleLineIndent = 0;
			}
		}
		for (int i = 0; i < indent; i++)
			pw.print('\t');
		for (int i = 0; i < singleLineIndent; i++)
			pw.print('\t');
		pw.println(s);
		if (s.endsWith("{")) {
			indent++;
		} else if (s.startsWith("else if ") || s.startsWith("if ") || s.startsWith("while ")
				|| s.startsWith("for ") || s.equals("else")) {
			singleLineIndent++;
		} else {
			singleLineIndent = 0;
		}
	}
}
