/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.globals;

import org.jabhts.annotation.global.TCompile;

public class ProgressInfo
{
	public final CompilerOptions options;
	public int numJavaSourcesUpdated, numTemplateDataUpdated;
	public int numErrors;
	public int numIgnoredSources, numNonCompiledTemplates, numCompiledTemplates;

	private boolean insideLine;


	public ProgressInfo(CompilerOptions options)
	{
		this.options = options;
	}

	public void printResult()
	{
		flush();
		System.err.println("Templates:     " + numCompiledTemplates + " compiled, "
				+ numNonCompiledTemplates + " not compiled, "
				+ numIgnoredSources + " files ignored.");
		System.err.println("Files updated: " + numTemplateDataUpdated + " templatedata, "
				+ numJavaSourcesUpdated + " java classes"
				+ (numJavaSourcesUpdated > 0 ? " !!!!!!!!!!!!!!!!!!!!" : ""));
		if (numErrors > 0)
			System.err.println(numErrors + " ERRORS");
	}

	public void scannedDirectory()
	{
		if (!options.verbose) {
			System.err.print("d");
			insideLine = true;
		}
	}

	public void startsFile(FileInfo info)
	{
		if (options.verbose) {
			flush();
			System.err.println("Compiles " + info.f.getName());
		} else {
			System.err.print("f");
			insideLine = true;
		}
	}

	public void startsTCompile(TCompile annot)
	{
		if (options.verbose) {
			flush();
			System.err.println("    tpname: " + annot.tpname());
		} else {
			System.err.print(".");
			insideLine = true;
		}
	}

	public void flush()
	{
		if (insideLine)
			System.err.println();
		insideLine = false;
	}
}
