/* Copyright (c) 2006-2011, Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package dk.vitality.lib.foundation.misc;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

@SuppressWarnings({ "UnusedDeclaration" })
public class ArrayUtils
{
	public static final Object[] emptyObjectArray = new Object[0];
	public static final boolean[] emptyBooleanArray = new boolean[0];
	public static final byte[] emptyByteArray = new byte[0];
	public static final char[] emptyCharArray = new char[0];
	public static final short[] emptyShortArray = new short[0];
	public static final int[] emptyIntArray = new int[0];
	public static final long[] emptyLongArray = new long[0];
	public static final String[] emptyStringArray = new String[0];
	public static final byte[][] emptyByteArrayArray = new byte[0][];

	@Nullable
	public static String[] concat(@Nullable String[] a, @Nullable String... b)
	{
		if (a == null || a.length == 0)
			return b;
		if (b == null || b.length == 0)
			return a;
		String[] c = new String[a.length + b.length];
		System.arraycopy(a, 0, c, 0, a.length);
		System.arraycopy(b, 0, c, a.length, b.length);
		return c;
	}

	@Nullable
	public static String[] concat(@Nullable String a, @Nullable String... b)
	{
		if (b == null || b.length == 0)
			return new String[] { a };
		String[] c = new String[1 + b.length];
		c[0] = a;
		System.arraycopy(b, 0, c, 1, b.length);
		return c;
	}

	@NotNull public static boolean[] realloc(@Nullable boolean[] src, int size)
	{
		boolean[] dst = new boolean[size];
		if (src != null && src.length > 0)
			System.arraycopy(src, 0, dst, 0, Math.min(src.length, size));
		return dst;
	}

	@NotNull public static int[] realloc(@Nullable int[] src, int size)
	{
		int[] dst = new int[size];
		if (src != null && src.length > 0)
			System.arraycopy(src, 0, dst, 0, Math.min(src.length, size));
		return dst;
	}

	@NotNull public static long[] realloc(@Nullable long[] src, int size)
	{
		long[] dst = new long[size];
		if (src != null && src.length > 0)
			System.arraycopy(src, 0, dst, 0, Math.min(src.length, size));
		return dst;
	}

	@NotNull public static String[] realloc(@Nullable String[] src, int size)
	{
		String[] dst = new String[size];
		if (src != null && src.length > 0)
			System.arraycopy(src, 0, dst, 0, Math.min(src.length, size));
		return dst;
	}
}
