/* Copyright (c) 2006-2011, Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package dk.vitality.lib.foundation.xml.xpath;

import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

public final class PathRuntime
{
	final Interest interest;
	final PathProgram program;
	private int currentComponentIndex;

	List<String> stringResult;
	List<Element> elementResult;
	int matchCount;

	enum Interest
	{
		Element, String, Count
	}

	public PathRuntime(PathProgram program, Interest interest)
	{
		this.program = program;
		this.interest = interest;
	}

	public void execute(Element contextElement) throws SAXException
	{
		currentComponentIndex = 0;
		if (program.paths.length == 0)
			return;
		program.paths[currentComponentIndex].execute(this, contextElement);
	}

	public void executeNextPathBase(PathBase currentComponent, Element currentElement) throws SAXException
	{
		ExpressionBase test = currentComponent.test;
		if (test != null) {
			String s = test.evaluate(this, currentElement);
			if (!stringToBoolean(s))
				return;
		}
		if (isLastComponent()) {
			collectElement(currentElement);
			return;
		}
		currentComponentIndex++;
		program.paths[currentComponentIndex].execute(this, currentElement);
		currentComponentIndex--;
	}

	public boolean isLastComponent()
	{
		return currentComponentIndex == program.paths.length - 1;
	}

	public void collectElement(Element e) throws SAXException
	{
		switch (interest) {
			case Count:
				matchCount++;
				return;
			case String:
				String s = XmlArgs.getOptNodeData(e);
				if (s != null && !s.isEmpty())
					doCollectString(s);
				return;
			case Element:
				if (elementResult == null)
					elementResult = new ArrayList<>();
				elementResult.add(e);
				break;
			default:
				throw new SAXException("Invalid interest = " + interest);
		}
	}

	public void collectString(String s) throws SAXException
	{
		switch (interest) {
			case Count:
				matchCount++;
				return;
			case String:
				doCollectString(s);
				return;
			case Element:
				throw new SAXException("Collected string when interested in elements!");
			default:
				throw new SAXException("Invalid interest = " + interest);
		}
	}

	private void doCollectString(String s)
	{
		if (stringResult == null)
			stringResult = new ArrayList<>();
		stringResult.add(s);
	}

	public static boolean stringToBoolean(String s)
	{
		return s != null && !s.isEmpty();
	}
}

// END OF FILE
