/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import "Rule.h"
#import "Xml.h"

@implementation Rule

+ (RuleType)getRuleType:(NSXMLElement*)ruleE
{
	NSString* type = [Xml getAttribute:ruleE withName:@"type"];
	if ([type isEqualToString:@"block"])
		return BlockRuleType;
	if ([type isEqualToString:@"modify"]) {
		if ([[Xml getAttribute:ruleE withName:@"content-type"] isEqual:@"*not-ok*"])
			return ModifyErrorStatusRuleType;
		else
			return ModifyContentRuleType;
	}
	if ([type isEqualToString:@"whitelist"])
		return WhitelistRuleType;
	if ([type isEqualToString:@"keyword"])
		return KeywordRuleType;
	if ([type isEqualToString:@"request"])
		return RequestRuleType;
	return UnknownRuleType;
}

+ (BOOL)ruleIsSafe:(NSXMLElement*)ruleE
{
	switch ([Rule getRuleType:ruleE]) {
		case WhitelistRuleType:
		case BlockRuleType:
			return YES;
		default:
			return NO;
	}
}

@end
