/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy;

import java.security.Security;
import org.glimmerblocker.proxy.globals.GlimmerGlobals;
import static dk.vitality.lib.foundation.exception.ExceptionUtils.getExMsg;

/**
 * Main class which runs the Grizzly server.
 */
public class GlimmerBlocker
{
	public static void main(String[] args)
	{
		try {
			String user = System.getProperty("user.name");
			if (user != null && !user.equals("_glimmerblocker")) {
				System.err.println("GlimmerBlockerProxy.app  should ONLY be launched by launchd!");
				System.err.println("Current user = '" + user + "'");
				if (System.getenv("GB_ALLOW_BAD_USERID") == null)
					return;
				// beware, this will trash subscriptions, and the Prefs Panel will not
				// be able to comm with the proxy as the proxy can't read the java settings with the secret key.
			}
			Security.setProperty("networkaddress.cache.ttl", "30");
			Security.setProperty("networkaddress.cache.negative.ttl", "10");
			//System.err.println("POS: " + InetAddressCachePolicy.get());
			//System.err.println("NEG: " + InetAddressCachePolicy.getNegative());
			boolean logToStderr = false;
			for (String s : args) {
				if (s == null)
					continue;
				if (s.equals("-stderr"))
					logToStderr = true;
				else if (!s.startsWith("--gb-version="))
					System.err.println("Launched with unknown argument: " + s);
			}
			new GlimmerGlobals(logToStderr);
		}
		catch (Exception ex) {
			System.err.println("GlimmerBlocker got exception in main thread: " + getExMsg(ex));
			ex.printStackTrace();
			System.exit(1);
		}
	}
}
