/* Copyright (C) 2009-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import dk.vitality.lib.foundation.exception.ExceptionUtils;
import dk.vitality.lib.foundation.misc.HexUtils;

/**
 * Just a simple test for checking if users can run Java.
 */
public class TestIPv6
{
	public static void main(String[] args)
	{
		System.err.println("preferIPv4 = " + getPrefer("java.net.preferIPv4Stack") +
				", preferIPv6 = " + getPrefer("java.net.preferIPv6Addresses"));
		System.err.println("java.version = " + System.getProperty("java.version"));
		System.err.println("os.version = " + System.getProperty("os.version"));
		System.err.println(System.getProperty("sun.arch.data.model") + "-bit jvm.");


		if (args.length == 0) {
			args = new String[]{
					"ipv6-test.com", "test-ipv6.com", "ipv6test.google.com"
			};
		}
		for (String s : args) {
			testHost(s);
		}
	}

	private static void testHost(String hostname)
	{
		System.err.println("===================================================================== " + hostname);
		System.err.println("Tests IPv6: " + hostname);
		InetAddress[] list;
		try {
			list = InetAddress.getAllByName(hostname);
		}
		catch (Throwable ex) {
			System.err.println("Got exception resolving name: " + ExceptionUtils.getExMsg(ex));
			return;
		}
		System.err.println("Got " + list.length + " addresses.");
		for (int i = 0; i < list.length; i++) {
			InetAddress addr = list[i];
			System.err.println("------ #" + (i + 1) + "/" + list.length);
			System.err.println("Got " + addr.getClass().getSimpleName() + ": " + addr);
			System.err.println("as bytes: " + HexUtils.toLcHex(addr.getAddress()));
			InetSocketAddress dst;
			try {
				dst = new InetSocketAddress(addr, 80);
			}
			catch (Throwable ex) {
				System.err.println("Got exception creating InetSocketAddress: " + ExceptionUtils.getExMsg(ex));
				continue;
			}
			System.err.println("InetSocketAddress: " + dst);
			Socket skt = new Socket();
			try {
				System.err.println("Got socket.");
				skt.connect(dst);
				System.err.println("Got connection.");
				System.err.println("Local port " + skt.getLocalPort() + " at " + skt.getLocalAddress());
			}
			catch (Throwable ex) {
				System.err.println("Got exception: " + ExceptionUtils.getExMsg(ex));
			}
			finally {
				try {
					skt.close();
				}
				catch (Throwable ex) {
					System.err.println("Got exception closing socket: " + ExceptionUtils.getExMsg(ex));
				}
			}
			System.err.println("Done testing connection.");
		}
	}

	private static String getPrefer(String name)
	{
		String s = System.getProperty(name);
		return (s == null || s.isEmpty()) ? "False" : s;
	}
}
