/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.globals.history;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import org.glimmerblocker.proxy.globals.GlimmerGlobals;
import org.glimmerblocker.proxy.rule.AbstractRule;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class HistoryCache
{
	@NotNull public final GlimmerGlobals globals;
	private boolean enabled;

	public HistoryCache(@NotNull GlimmerGlobals globals)
	{
		this.globals = globals;
	}

	public void loadSettings() throws Exception
	{
		synchronized (cache) {
			boolean wantsEnabled = globals.panelSettingsXA.getOptBoolean("history/@enabled");
			if (enabled == wantsEnabled)
				return;
			globals.ln("HistoryCache.enabled: " + enabled + " -> " + wantsEnabled);
			enabled = wantsEnabled;
			if (!enabled)
				cache.clear();
		}
	}

	public HistoryEntryBuilder makeBuilder(@NotNull String url, @Nullable String referer)
	{
		if (enabled)
			return new HistoryEntryBuilderImpl(this, url, referer);
		else
			return dummyBuilder;
	}

	public void addEntry(HistoryEntry entry)
	{
		if (!enabled)
			return;
		if (globals.debugVerbose)
			globals.ln("HistoryCache.adds: " + entry.actions + "; " + entry.url);
		synchronized (cache) {
			cache.put(entry, entry);
		}
	}

	@NotNull
	public List<HistoryEntry> getEntries()
	{
		synchronized (cache) {
			return new ArrayList<>(cache.values());
		}
	}

	private static final HistoryEntryBuilder dummyBuilder = new HistoryEntryBuilder()
	{
		public void addAction(@NotNull AbstractRule rule, @NotNull String action)
		{
		}

		public void commit()
		{
		}
	};

	private final LruCache cache = new LruCache();
	private static final int cacheSize = 150;

	private static class LruCache extends LinkedHashMap<HistoryEntry, HistoryEntry>
	{
		private LruCache()
		{
			super(cacheSize);
		}

		@Override
		protected boolean removeEldestEntry(Map.Entry<HistoryEntry, HistoryEntry> eldest)
		{
			return size() >= cacheSize;
		}
	}
}
