/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.globals.log;

import java.text.SimpleDateFormat;
import java.util.Date;
import org.jetbrains.annotations.Nullable;

/**
 * Simple Logger implementation which writes the message to System.err prepended by a timestamp.
 */
public class DatePrefixedLogger implements Logger
{
	public void ln(@Nullable String msg)
	{
		if (msg == null)
			return;
		String d = getDatePrefix();
		if (msg.indexOf('\n') < 0) {
			System.err.println(d + " " + msg);
			return;
		}
		int len = msg.length();
		if (len == 0) {
			System.err.println(d);
			return;
		}
		int numLines = 1;
		for (int i = 0; i < len; i++) {
			if (msg.charAt(i) == '\n')
				numLines++;
		}
		StringBuilder sb = new StringBuilder(numLines * (d.length() + 2) + msg.length());
		int start = 0;
		while (start < len) {
			sb.append(d);
			int idx = msg.indexOf('\n', start);
			if (idx < 0)
				idx = len;
			if (idx > start)
				sb.append(' ').append(msg.substring(start, idx)).append('\n');
			start = idx + 1;
		}
		System.err.print(sb.toString());
	}

	public static String getDatePrefix()
	{
		synchronized (logDateFmt) {
			return logDateFmt.format(new Date());
		}
	}

	private static final SimpleDateFormat logDateFmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
}
