/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.globals.suspect;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Immutable.  Use as both data and the key into the LRU cache map.
 */
public class Suspect
{
	@NotNull public final String host;
	@NotNull public final String path;
	@NotNull public final String query;
	public final boolean crossDomain;
	@Nullable public final String referer;
	@NotNull public final SuspectType type;
	@NotNull public final String description;

	public Suspect(@NotNull String host,
	               @NotNull String path,
	               @NotNull String query,
	               boolean crossDomain,
	               @Nullable String referer,
	               @NotNull SuspectType type,
	               @NotNull String description)
	{
		this.host = host;
		this.path = path;
		this.query = query;
		this.crossDomain = crossDomain;
		this.referer = referer;
		this.type = type;
		this.description = description;
	}

	/**
	 * partial: uses only host+path+query for key into Map/Sets.
	 */
	@Override
	public boolean equals(Object o)
	{
		if (this == o)
			return true;
		if (o == null || getClass() != o.getClass())
			return false;
		Suspect o2 = (Suspect)o;
		return host.equals(o2.host) && path.equals(o2.path) && query.equals(o2.query);
	}

	/**
	 * partial: uses only host+path+query for key into Map/Sets.
	 */
	@Override
	public int hashCode()
	{
		return (host.hashCode() * 31 + path.hashCode()) * 31 + query.hashCode();
	}
}
