/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.client.curl;

import java.io.IOException;
import java.net.InetAddress;
import java.net.URL;
import org.glimmerblocker.proxy.globals.GlimmerGlobals;
import org.glimmerblocker.proxy.globals.log.Logger;
import org.glimmerblocker.proxy.globals.network.NetworkConfig;
import org.glimmerblocker.proxy.globals.network.NetworkConfigSubObject;
import org.glimmerblocker.proxy.http.data.Headers;
import org.glimmerblocker.proxy.rhino.callback.CallbackRhinoBinder;
import org.glimmerblocker.proxy.rhino.callback.ContentGlobalObjectBase;
import org.glimmerblocker.proxy.rhino.callback.RhinoCallback;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;
import dk.vitality.lib.foundation.ref.FatRef;

public class RequestGlobalObject extends ContentGlobalObjectBase
{
	public InetAddress clientAddr;
	public String method;
	public URL url;
	public Headers origHeaders;
	public FatRef<NetworkConfig> networkConfigRef;
	public RequestSubObject request;

	public NetworkConfigSubObject network;

	private static final CallbackRhinoBinder classBinder = new CallbackRhinoBinder(RequestGlobalObject.class);

	public RequestGlobalObject(@NotNull GlimmerGlobals globals,
	                           @NotNull Scriptable scope,
	                           @NotNull Scriptable prototype) throws IOException
	{
		this(globals, scope, prototype, classBinder);
	}

	public RequestGlobalObject(@NotNull GlimmerGlobals globals, @NotNull Scriptable scope,
	                           @NotNull Scriptable prototype,
	                           @NotNull CallbackRhinoBinder binder) throws IOException
	{
		super(globals, scope, prototype, binder);
	}

	public void prepareRequestObject(@NotNull Context ctx,
	                                 @NotNull Logger logger,
	                                 @Nullable InetAddress clientAddr,
	                                 @NotNull String method,
	                                 @NotNull URL url,
	                                 @NotNull Headers origHeaders,
	                                 @NotNull FatRef<NetworkConfig> networkConfigRef) throws Exception
	{
		super.prepareContentCallback(logger, ctx, null, Headers.simpleTextHtml);
		this.clientAddr = clientAddr;
		this.method = method;
		this.url = url;
		this.origHeaders = origHeaders;
		this.networkConfigRef = networkConfigRef;
		if (network != null)
			prepareNetworkConfigSubObject();
		if (request != null)
			request.prepareRequestSubObjectBase(ctx, clientAddr, method, url, origHeaders);
	}

	@Override
	public void notifyLeftContext()
	{
		super.notifyLeftContext();
		if (network != null)
			network.notifyLeftContext();
		if (request != null)
			request.notifyLeftContext();
	}

	@Nullable
	protected URL getCurlBaseUrl()
	{
		return url;
	}
	//===========================================================================

	@RhinoCallback
	public NetworkConfigSubObject jsGet_network() throws Exception
	{
		if (network == null) {
			network = new NetworkConfigSubObject(this);
			network.bind();
			prepareNetworkConfigSubObject();
		}
		return network;
	}

	private void prepareNetworkConfigSubObject() throws Exception
	{
		network.prepareNetworkConfigSubObject(ctx, networkConfigRef);
	}

	//===========================================================================

	@RhinoCallback
	public RequestSubObject jsGet_request() throws Exception
	{
		if (request == null) {
			request = createRequestSubObject();
			request.bind();
			request.prepareRequestSubObjectBase(ctx, clientAddr, method, url, origHeaders);
		}
		return request;
	}

	protected RequestSubObject createRequestSubObject() throws Exception
	{
		return new RequestSubObject(this);
	}

	@NotNull
	@Override
	protected StaticRequestSubObject getRequestObject() throws Exception
	{
		return jsGet_request();
	}

	//===========================================================================
}
