/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.client.curl;

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.regex.Pattern;
import org.glimmerblocker.proxy.http.data.Headers;
import org.glimmerblocker.proxy.rhino.callback.CallbackObjectBase;
import org.glimmerblocker.proxy.rhino.callback.CallbackRhinoBinder;
import org.glimmerblocker.proxy.rhino.callback.RhinoCallback;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.mozilla.javascript.Context;

public class RequestSubObject extends StaticRequestSubObject
{
	private static final CallbackRhinoBinder classBinder = new CallbackRhinoBinder(RequestSubObject.class);

	public RequestSubObject(@NotNull CallbackObjectBase parent)
	{
		super(parent, classBinder);
	}

	public RequestSubObject(@NotNull CallbackObjectBase parent, @NotNull CallbackRhinoBinder binder)
	{
		super(parent, binder);
	}

	public void prepareRequestSubObjectBase(@NotNull Context ctx,
	                                        @Nullable InetAddress clientAddr,
	                                        @NotNull String method,
	                                        @NotNull URL url,
	                                        @NotNull Headers origHeaders) throws Exception
	{
		super.prepareStaticRequestSubObject(ctx, clientAddr, method, url, origHeaders);
	}

	protected void sealHeadersMapObject()
	{
		// no sealing as it's not readonly.
	}

	private void setUrl(URL url)
	{
		this.url = url;
		jsGet_headers().setHeaderLC("host", url.getHost());
	}

	//===========================================================================

	@RhinoCallback
	public void jsSet_method(String s) throws Exception
	{
		if (s == null || s.length() == 0)
			throw Context.reportRuntimeError("HTTP method must not be null/empty string.");
		s = s.toUpperCase();
		if (s.equals("CONNECT"))
			throw Context.reportRuntimeError("CONNECT method is not supported.");
		if (!validMethodRE.matcher(s).matches())
			throw Context.reportRuntimeError("Invalid HTTP method: '" + s + "'.");
		method = s;
	}

	private static final Pattern validMethodRE = Pattern.compile("^[A-Z]{1,100}$");

	//===========================================================================

	@RhinoCallback
	public void jsSet_url(String s) throws MalformedURLException
	{
		setUrl(new URL(s));
	}

	@RhinoCallback
	public void jsSet_urlHost(String host) throws MalformedURLException
	{
		setUrl(new URL(url.getProtocol(), host, url.getPort(), url.getPath()));
	}

	@RhinoCallback
	public void jsSet_urlPort(int port) throws MalformedURLException
	{
		setUrl(new URL(url.getProtocol(), url.getHost(), port, url.getPath()));
	}

	@RhinoCallback
	public void jsSet_urlHostPort(String hp) throws MalformedURLException
	{
		int idx = hp.indexOf(':');
		String host = (idx < 0) ? hp : hp.substring(0, idx);
		int port = (idx < 0 || idx == hp.length() - 1) ? 0 : Integer.parseInt(hp.substring(idx + 1));
		setUrl(new URL(url.getProtocol(), host, port, url.getPath()));
	}

	@RhinoCallback
	public void jsSet_urlPath(String path) throws MalformedURLException
	{
		if (path == null || path.length() == 0)
			path = "/";
		String q = url.getQuery();
		String f = (q != null) ? path + "?" + q : path;
		setUrl(new URL(url.getProtocol(), url.getHost(), url.getPort(), f));
	}

	@RhinoCallback
	public void jsSet_urlQuery(String q) throws MalformedURLException
	{
		String f = jsGet_urlPath();
		if (q != null && q.length() > 0)
			f += "?" + q;
		setUrl(new URL(url.getProtocol(), url.getHost(), url.getPort(), f));
	}

	//===========================================================================

	@RhinoCallback
	public void jsSet_contentType(String s)
	{
		jsGet_headers().setHeaderLC("content-type", s);
	}

	@RhinoCallback
	public void jsSet_userAgent(String s)
	{
		jsGet_headers().setHeaderLC("user-agent", s);
	}

	@RhinoCallback
	public void jsSet_referer(String s)
	{
		jsGet_headers().setHeaderLC("referer", s);
	}
}
