/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.proxyRequest.transform;

import java.io.IOException;
import java.net.URL;
import org.glimmerblocker.proxy.globals.GlimmerGlobals;
import org.glimmerblocker.proxy.globals.log.Logger;
import org.glimmerblocker.proxy.http.client.curl.CurlBase;
import org.glimmerblocker.proxy.http.client.curl.StaticRequestSubObject;
import org.glimmerblocker.proxy.http.data.Headers;
import org.glimmerblocker.proxy.proxyRequest.ProxyRequestHandler;
import org.glimmerblocker.proxy.rhino.callback.CallbackRhinoBinder;
import org.glimmerblocker.proxy.rhino.callback.ContentGlobalObjectBase;
import org.glimmerblocker.proxy.rhino.callback.RhinoCallback;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;

@SuppressWarnings({"serial", "NonSerializableFieldInSerializableClass", "UnusedDeclaration"})
public class TransformerGlobalObject extends ContentGlobalObjectBase
{
	ProxyRequestHandler prh;
	CurlBase curlBase;

	StaticRequestSubObject request;

	private static final CallbackRhinoBinder classBinder = new CallbackRhinoBinder(TransformerGlobalObject.class);

	public TransformerGlobalObject(@NotNull GlimmerGlobals globals,
	                               @NotNull Scriptable scope,
	                               @NotNull Scriptable prototype) throws IOException
	{
		super(globals, scope, prototype, classBinder);
	}

	public void prepareTransformerCallback(@NotNull Logger logger,
	                                       @NotNull Context ctx,
	                                       @Nullable String t,
	                                       @NotNull ProxyRequestHandler prh,
	                                       @NotNull CurlBase curlBase,
	                                       @NotNull Headers responseHeaders) throws Exception
	{
		super.prepareContentCallback(logger, ctx, t, responseHeaders);
		this.prh = prh;
		this.curlBase = curlBase;
		if (request != null)
			prepareRequestObject();
	}

	@Override
	public void notifyLeftContext()
	{
		super.notifyLeftContext();
		prh = null;
		curlBase = null;
	}

	@Nullable
	protected URL getCurlBaseUrl()
	{
		return curlBase.url;
	}
	//===========================================================================

	@RhinoCallback
	public StaticRequestSubObject jsGet_request() throws Exception
	{
		if (request == null) {
			request = new StaticRequestSubObject(this);
			request.bind();
			prepareRequestObject();
		}
		return request;
	}

	private void prepareRequestObject() throws Exception
	{
		request.prepareStaticRequestSubObject(ctx,
				prh.request.getClientAddress(),
				curlBase.method, curlBase.url, curlBase.requestHeaders);
	}

	@NotNull
	@Override
	protected StaticRequestSubObject getRequestObject() throws Exception
	{
		return jsGet_request();
	}

	//===========================================================================
	// OBSOLETE METHODS, only for compability with GB 1.3

	@Nullable
	@RhinoCallback
	public String jsGet_contentType() throws Exception
	{
		return jsGet_response().jsGet_contentType();
	}

	@Nullable
	@RhinoCallback
	public String jsGet_referer()
	{
		return prh.request.headers.getLC("referer");
	}

	@RhinoCallback
	public String jsGet_url() throws Exception
	{
		return jsGet_request().jsGet_url();
	}

	@RhinoCallback
	public String jsGet_urlpath() throws Exception
	{
		return jsGet_request().jsGet_urlPath();
	}

	@RhinoCallback
	public String jsGet_urlquery() throws Exception
	{
		return jsGet_request().jsGet_urlQuery();
	}

	@RhinoCallback
	public String jsGet_method() throws Exception
	{
		return jsGet_request().jsGet_method();
	}
}
