// This is a function which is compiled by org.mozilla.javascript.Context.compileFunction()

// http://kangax.github.com/es5-compat-table/

function stringify(obj)
{
	var sb = [];
	var escapes = {
		'\b': '\\b',
		'\t': '\\t',
		'\r': '\\r',
		'\n': '\\n',
		'\f': '\\f',
		'\\': '\\\\',
		'\'' : '\\\''
	};
	var handleString = function(s)
	{
		//noinspection JSUnusedLocalSymbols
		s = s.replace(/([\x00-\x1f\\'])/g, function(a, b)
		{
			var c = escapes[b];
			if (c)
				return c;
			c = b.charCodeAt(0);
			return '\\x' + Math.floor(c / 16).toString(16) + (c % 16).toString(16);
		});
		sb.push("'");
		sb.push(s);
		sb.push("'");
	};
	var f = function(obj)
	{
		// http://javascriptweblog.wordpress.com/2011/08/08/fixing-the-javascript-typeof-operator/
		if (obj === null) {
			sb.push('null');
			return;
		}
		var t = typeof(obj);
		if (t === "undefined") {
			sb.push('undefined');
			return;
		}
		if (t === 'boolean') {
			sb.push(obj ? "true" : "false");
			return;
		}
		if (t === 'null') {
			sb.push('null');
			return;
		}
		if (t === 'number') {
			sb.push(obj.toString());
			return;
		}
		if (Array.isArray(obj)) {
			sb.push('[');
			var len = obj.length;
			for (var i = 0; i < len; i++) {
				if (i > 0)
					sb.push(', ');
				f(obj[i]);
			}
			sb.push(']');
			return;
		}
		if (t === 'object') {
			var tt = ({}).toString.call(obj).match(/\s([a-z|A-Z]+)/)[1].toLowerCase();
			switch (tt) {
				case 'date':
					sb.push("Date{");
					sb.push(obj.toString());
					sb.push("}");
					return;
				case 'regexp':
				case 'number':
				case 'boolean':
					sb.push(obj.toString());
					return;
				case 'string':
					handleString(obj);
					return;
			}
			var num = 0;
			for (var attr in obj) {
				sb.push(num++ ? ', ' : '{ ');
				if (attr.match(/^[a-zA-Z][_\w]+$/))
					sb.push(attr);
				else
					sb.push("'" + attr + "'");
				sb.push(':');
				f(obj[attr]);
			}
			sb.push(num ? ' }' : '{}');
			return;
		}
		if (t === 'string') {
			handleString(obj);
			return;
		}
		if (t === 'function') {
			sb.push(obj.toSource());
			return;
		}
		sb.push(' /* UNKNOWN OBJECT TYPE: ' + t + ' */ ');
		//sb.push(obj.toSource());
	};
	f(obj);
	return sb.join('');
}
