/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rule;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public enum HostMatchType
{
	// least specific first, and most specific last
	Ignored()
			{
				public String debugDescription(@Nullable String value)
				{
					return "-";
				}
				public String userDescription(@Nullable String value)
				{
					return "host ignored";
				}
			},
	Regexp()
			{
				public String debugDescription(@Nullable String value)
				{
					return "regexp{" + value + "}";
				}
				public String userDescription(@Nullable String value)
				{
					return "host matches regexp: " + value;
				}
			},
	Domain()
			{
				public String debugDescription(@Nullable String value)
				{
					return "*." + value;
				}
				public String userDescription(@Nullable String value)
				{
					return "host is in domain: " + value;
				}
			},
	Is()
			{
				public String debugDescription(@Nullable String value)
				{
					return value;
				}
				public String userDescription(@Nullable String value)
				{
					return "host is: " + value;
				}
			};

	public abstract String debugDescription(@Nullable String value);

	public abstract String userDescription(@Nullable String value);

	@NotNull
	public static HostMatchType fromXmlValue(@Nullable String v) throws IllegalArgumentException
	{
		if (v == null)
			return Is;
		if (v.equals("regexp"))
			return Regexp;
		if (v.equals("domain"))
			return Domain;
		if (v.equals("is"))
			return Is;
		throw new IllegalArgumentException("Invalid host-type='" + v + "'");
	}
}
