/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rule;

import org.glimmerblocker.proxy.proxyRequest.transform.MimeTypeMatcher;
import org.glimmerblocker.proxy.rhino.runtime.RhinoRuntimeCache;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import dk.vitality.lib.foundation.xml.xpath.XmlArgs;

public class ModificationRule extends AbstractRule
{
	@Nullable public final String css;
	@Nullable public final String cssElement;

	@Nullable public final String js;
	@Nullable public final String jsElement;
	@Nullable public final ElementPlacementType jsPlacement;

	@Nullable public final String transform;
	@Nullable public final MimeTypeMatcher transformContentTypeMatcher;

	public final int runtimeId;

	public ModificationRule(boolean isPreview,
	                        int filterId,
	                        @NotNull String filterName,
	                        @NotNull XmlArgs ruleXA) throws Exception
	{
		super(isPreview, filterId, filterName, ruleXA, RuleType.Modification);
		//
		XmlArgs cssXA = ruleXA.getOptXmlArgs("css");
		if (cssXA != null) {
			css = cssXA.getOptEmptyString(".");
			cssElement = "\n\n" +
					"<style type='text/css'> <!--\n" +
					"/* GlimmerBlocker whitelist styles */\n" +
					css.trim() + "\n" +
					"-->\n" +
					"</style>\n\n";
		} else {
			css = "";
			cssElement = null;
		}
		//
		XmlArgs jsXA = ruleXA.getOptXmlArgs("js");
		if (jsXA != null) {
			js = jsXA.getOptEmptyString(".");
			jsElement = "\n\n" +
					"<script type='text/javascript'><!--\n" +
					"// GlimmerBlocker whitelist script\n" +
					js.trim() + "\n" +
					"//-->\n" +
					"</script>\n\n";
			jsPlacement = ElementPlacementType.fromXmlValue(jsXA.getOptString("@placement"));
		} else {
			js = "";
			jsElement = null;
			jsPlacement = null;
		}
		//
		XmlArgs transformXA = ruleXA.getOptXmlArgs("transform");
		if (transformXA != null) {
			this.transform = transformXA.getReqString(".");
			String ct = transformXA.getOptString("@content-type");
			this.transformContentTypeMatcher = MimeTypeMatcher.fromXml(ct);
			this.runtimeId = RhinoRuntimeCache.makeNewRuntimeId();
		} else {
			this.transform = null;
			this.transformContentTypeMatcher = null;
			this.runtimeId = 0;
		}
	}

	/**
	 * Do NOT rely on the contents of this string for anything. It is only for debugging.
	 */
	public String toString()
	{
		StringBuilder sb = new StringBuilder(100);
		if (css != null)
			sb.append("Css");
		if (js != null)
			sb.append("Js");
		if (transform != null)
			sb.append("Transform");
		sb.append("{");
		toStringMatching(sb);
		sb.append("}");
		appendFilterName(sb);
		if (transformContentTypeMatcher != null)
			sb.append("{").append(transformContentTypeMatcher.toString()).append("}");
		return sb.toString();
	}
}
