/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rule;

import java.util.HashMap;
import java.util.Map;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public enum QueryMatchType
{
	// least specific first, and most specific last
	Ignored(null) {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "-";
		}

		@NotNull
		public String userDescription(@Nullable String host)
		{
			return "query ignored";
		}
	},
	Regexp("regexp") {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "regexp{" + value + "}";
		}

		@NotNull
		public String userDescription(@Nullable String value)
		{
			return "query matches regexp: " + value;
		}
	},
	NotRegexp("not-regexp") {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "not-regexp{" + value + "}";
		}

		@NotNull
		public String userDescription(@Nullable String value)
		{
			return "query does not match regexp: " + value;
		}
	},
	Contains("contains") {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "contains{" + value + "}";
		}

		@NotNull
		public String userDescription(@Nullable String value)
		{
			return "query contains: " + value;
		}
	},
	NotContains("not-contains") {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "not-contains{" + value + "}";
		}

		@NotNull
		public String userDescription(@Nullable String value)
		{
			return "query does not contain: " + value;
		}
	};

	@Nullable
	public final String xmlName;

	QueryMatchType(@Nullable String xmlName)
	{
		this.xmlName = xmlName;
	}

	private static final Map<String, QueryMatchType> xmlMap = new HashMap<>();

	static {
		for (QueryMatchType t : values())
			xmlMap.put(t.xmlName, t);
	}

	@NotNull
	public abstract String debugDescription(@Nullable String value);

	@NotNull
	public abstract String userDescription(@Nullable String value);

	public static QueryMatchType fromXmlValue(@Nullable String xmlName) throws IllegalArgumentException
	{
		if (xmlName == null)
			return Contains;
		QueryMatchType t = xmlMap.get(xmlName);
		if (t != null)
			return t;
		throw new IllegalArgumentException("Invalid query-type='" + xmlName + "'");
	}
}
