#!/usr/bin/perl

use strict;

my $fn = "./bin/cert-vars.txt";
open(FH, $fn) || die "Can't open $fn";
$_ = join("", <FH>);
close(FH);

unless (/^CERT_IDENTIFIER=(\w+)\s*$/m) {
	die "Missing CERT_IDENTIFIER=...40.hex.digits... in $fn";
}
my $cert = $1;

unless (/^CERT_DEVELOPER=(\w+)\s*$/m) {
	die "Missing CERT_DEVELOPER=..10.chars.developer.id.. in $fn";
}
my $developer = $1;

unless (@ARGV) {
	die "Missing bundle path args";
}

sub sign
{
	my ($bundleId, $path) = @_;
	my (@cmd) = ("codesign");
	if ($bundleId eq "org.glimmerblocker.proxyApp") {
		push(@cmd, "--deep");
	}
	if ($bundleId eq "<org.glimmerblocker.prefsPane") {
		push(@cmd, "--force", "--deep");
	}
	push(@cmd,
			"--sign", $cert,
			"--verbose=2",
#"--deep",  # makes Xcode 6.3 fail.
			"--identifier", $bundleId,
			"--requirements", "=designated => anchor apple generic and identifier \"$bundleId\" and ( (cert leaf[field.1.2.840.113635.100.6.1.9] exists) or (certificate 1[field.1.2.840.113635.100.6.2.6] exists and certificate leaf[field.1.2.840.113635.100.6.1.13] exists and certificate leaf[subject.OU] = \"$developer\"))",
			$path);
	print "# ", join(" ", map { / / ? "'$_'" : $_; } @cmd), "\n";
	my $x = system(@cmd);
	if ($x) {
		die "codesign failed with exitcode $x = ".($x / 256).", ? = $?\n";
	}
}

foreach my $bundlePath (@ARGV) {
	if (! -d $bundlePath) {
		die "Missing dir $bundlePath";
	}
	$fn = "$bundlePath/Contents/Info.plist";
	open(FH, $fn) || die "Can't open $fn";
	$_ = join("", <FH>);
	close(FH);
	unless (m{<key>CFBundleIdentifier</key>\s*<string>(\S+)</string>}) {
		print;
		die "Can't find CFBundleIdentifier in $fn";
	}
	my $bundleId = $1;
	if ($bundleId eq "org.glimmerblocker.installer") {
		sign($bundleId, "$bundlePath/Contents/MacOS/InstallerDaemon");
	}
	sign($bundleId, $bundlePath);
}
